//////////////////////////////////////////////////
////Shader for Rendering the Depthmap for VSM/////
//////////////////////////////////////////////////
//////////////////////////////////////////////////
////14.01.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

// Constants given by the engine
float4x4 matWorld;
float4x4 matMtl;

float4 vecProjViewPos;
float4 vecValues;

// Textures
texture entSkin1;
sampler ColorMapSampler = sampler_state
{
	Texture = <entSkin1>;
	MipFilter = LINEAR;
};

// Structs
struct SMapVertexToPixel
{
    float4 Position     : POSITION;
    float Depth		   : TEXCOORD0;
    float2 TexCoords    : TEXCOORD1;
};

// Vertex Shader
SMapVertexToPixel ShadowMapVertexShader( float4 inPos : POSITION, float2 texcoords : TEXCOORD0)
{
    SMapVertexToPixel Output;
    
    Output.Position = mul(inPos,mul(matWorld,matMtl));
    
///////////////////////////////////////////////////////////////////////////
	// Verzerrungszentrum wieder in gSchattenBetrachterPos gegeben, in .zw liegen wieder die Konstanten
	// Abstand zum Verzerrungszentrum
	float2 diff = Output.Position.xy - vecProjViewPos.xy;
	float t = length(diff)*0.002;
	
	// Verzerrung anwenden
	float nt = vecProjViewPos.w * log( vecProjViewPos.z * t + 1) / t;
	
	// in neue Vertexposition umrechnen
	Output.Position.xy = vecProjViewPos.xy + diff * nt;
///////////////////////////////////////////////////////////////////////////

    Output.Depth = Output.Position.z;

    Output.TexCoords = texcoords;
    
    return Output;
}

// Pixel Shader
float4 ShadowMapPixelShader(SMapVertexToPixel PSIn) : COLOR0
{
	float4 Color = PSIn.Depth;
	Color.g = PSIn.Depth*PSIn.Depth;
	Color.a = 1.0; //tex2D(ColorMapSampler,PSIn.TexCoords).a;
	return Color;
}

// Technique
technique ShadowMap
{
	pass Pass0
	{
		ZWriteEnable = True;
		AlphaBlendEnable = False;
		AlphaTestEnable = True;
		AlphaRef = 97;
		AlphaFunc = Greater;
		
		CullMode = CCW;
		
		VertexShader = compile vs_1_1 ShadowMapVertexShader();
		PixelShader = compile ps_1_1 ShadowMapPixelShader();
	}
}