////////////////////////////////////////////////////
// Poisson blur shader
// Copyright (c) 2007 Conitec. All rights reserved.
////////////////////////////////////////////////////

//Tweakables
static const float fScale = 6.0;

// Application fed data:
float4 vecViewPort;

texture TargetMap;
texture StencilMap;
sampler ColorSampler = sampler_state { Texture = <TargetMap>; };
sampler ShadowSampler = sampler_state { Texture = <StencilMap>; };
float4 vecSkill1;

#define NUM_TAPS 12

static const float2 fTaps_Poisson[NUM_TAPS] = {
	{-.326,-.406},
	{-.840,-.074},
	{-.696, .457},
	{-.203, .621},
	{ .962,-.195},
	{ .473,-.480},
	{ .519, .767},
	{ .185,-.893},
	{ .507, .064},
	{ .896, .412},
	{-.322,-.933},
	{-.792,-.598}};

// Poisson blur pixel shader
float4 PoissonPS (in float2 inShadow: TEXCOORD0) : COLOR0
{
	float fShadow = 0.0;
	for (int i=0; i < NUM_TAPS; i++)
	{
		fShadow += tex2D(ShadowSampler,inShadow + fTaps_Poisson[i]*vecSkill1.x*vecViewPort.zw).a;
	}
	fShadow *= 1.0/NUM_TAPS;
	return tex2D(ColorSampler,inShadow)-vecSkill1.y*fShadow;
}

technique techPoisson
{
	pass p0
	{
		PixelShader = compile ps_2_0 PoissonPS();
	}
}
